%===================================%
%  Martin Bernasconi                %
%                                   %
%  TU Darmstadt                     %
%                                   %
%  Fuzzy-C-Means GUI V1.4           %
%===================================%
%
% 
% Bedienung der Fuzzy-C-Means-GUI
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% 
% Die Bedieneroberflche unterteilt sich in 5 Hauptfelder:
% 
% 0.	dem Visualisierungsfenster
% 1.	dem Fenster fr Einstellungen
% 2.	dem Fenster fr die Punkterzeugung
% 3.	dem Fenster fr die Clusterberechnung
% 4.	dem Fenster fr die Ausgabensteuerung in der Konsole
% 
% 
% 0. Visualisierung
% Die Visualisierung der Punkte erfolgt in diesem Fenster durch Sterne. 
% Die Zugehrigkeit zu den einzelnen Clustern ist erkennbar durch die 
% Farbe des Punktes. Je satter die Farbe, desto eindeutiger gehrt der 
% Punkt zu dem betreffenden Cluster. Ist der Punkt nur gering einem Cluster 
% zugeordnet, ist der Punkt umso schwrzer.
% 
% Die Visualisierung der Clusterpunkte erfolgt in diesem Fenster durch Kreise. 
% Sind aus vorangegangenen Berechnungen alte Clusterpunkte vorhanden, werden 
% diese mitgeplottet. Der Clusterverlauf ist gekennzeichnet durch Linien, 
% die die Clusterpunkte verbinden.
% 
% 
% 1. Einstellungen
% 
% "Anzahl Cluster" am Slider einstellen. Wird die Anzahl der Cluster 
% verndert, wird Zugehrigkeitsmatrix U resettet und alle schon 
% berechneten Clustermittelpunkte auf Null gesetzt.
% 
% "Kontrast alpha" am Slider einstellen. Kann zwischen Berechnungen 
% verndert werden, fhrt aber nicht mehr zu anschaulichen Ergebnissen.
% 
% "x-Achse" und "y-Achse" am Slider variieren. Kann benutzt werden, um 
% in kleineren Dimensionen zu arbeiten. Kann whrend Berechnungen verndert 
% werden.
%  
% "Farben invertieren" invertiert alle verwendeten Farben. Kann Erkennbarkeit 
% erhhen.
% 
% "Gitter ein/aus" schaltet ein Koordinatengitter ein/aus.
% 
% 
% 2. Punkte
% 
% "per Zufall setzen" setzt die am Slider eingestellte Anzahl neue Punkte. 
% Die neuen Punkte werden schon vorhandenen Punkten hinzugefgt.
% 
% Mit "per Maus setzten" kann man im Koordinatenfenster mit der Maus Punkte 
% hinzufgen. Unter dem Button steht, wie man das Setzten beendet.
% 
% Mit "Punkte laden" knnen Sie eine *.mat-Datei einlesen. Das Format dieser 
% Datei ist allerdings fest vorgegeben. Eine verwendbare *.mat-Datei beinhaltet 
% als Variablen:
% 
% m       eine (2 x #Punkte)-Punktmatrix, deren x- und y-Werte aus dem 
%         Intervall [0, 10] sind
% c       Anzahl der Cluster aus dem Intervall [2, 10]
% error   der Abbruchfehler aus dem Intervall [0.001, 0.1]
% alpha   der Kontrastparameter aus dem Intervall [1.01, 10]
% 
% "Punkte speichern" speichert die in Punkte laden genannten Parameter, 
% wie sie im Moment in der GUI eingestellt sind.
% 
% "alle Punkte loeschen" lscht alle vorhandenen Punkte und Clusterpunkte.
% 
% 
% 3. Cluster
% 
% "Berechne einen Schritt" berechnet mit der zurzeit aktuellen 
% Zugehrigkeitsmatrix U einen Iterationsschritt des Fuzzy-C-Means-Algorithmus. 
% Die neuen Clusterpunkte werden als rote Kreise im Koordinatenfenster angezeigt, 
% wobei ltere Clusterpunkte erhalten bleiben. Eine rote Pfadlinie zeigt an, 
% welcher Clusterpunkt sich wohin bewegt hat.
% 
% "Berechne n Schritte" berechnet mit der zurzeit aktuellen Zugehrigkeitsmatrix U 
% eine am Slider eingestellte Anzahl Iterationsschritte des Fuzzy-C-Means-Algorithmus. 
% Anzeige wie in "Berechne einen Schritt".
% 
% "Rechne bis Abbruch" berechnet Clusterpunkte via Fuzzy-C-Means-Algorithmus, 
% bis der am Slider eingestellte Abbruchfehler unterboten ist. Beim Aufruf 
% wird jeweils eine neue zufllige Zugehrigkeitsmatrix gebildet, die als 
% aktuelle erhalten bleibt. Die Anzahl der bentigten Schritte wird in 
% "aktueller Schritt" angezeigt.
% 
% "Clusterpunkte loeschen" lscht alle im Koordinatenfenster angezeigten 
% Clusterpunkte.
% 
% "Matrix U loeschen" ersetzt die aktuelle Zugehrigkeitsmatrix U mit einer neuen 
% zufllig generierten Matrix. Hiernach hat man wieder die Mglichkeit, mit "Berechne 
% einen Schritt" und "Berechne n Schritte" sich einen neuen Clusterpunkte-Weg schrittweise 
% auszurechnen.
% 
% 
% 4. Ausgaben in die Konsole
% 
% Die hier anzuwhlenden Daten werden nach jeweils einem Berechnungsklick in der 
% Konsole angegeben.
% 
% 
% Falls Fehler bemerkt werden, bitte ich darum, mich unter der oben genannten 
% EMail-Addresse zu kontaktieren. Dann werde ich Fehler schnellstmglich beheben.
% Vielen Dank und viel Spa beim Experimentieren.




function varargout = fuzzyCMeansGUI(varargin)

% initialisierung
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @fuzzyCMeansGUI_OpeningFcn, ...
                   'gui_OutputFcn',  @fuzzyCMeansGUI_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% ende initialisierung


% --- Executes just before fuzzyCMeansGUI is made visible.
function fuzzyCMeansGUI_OpeningFcn(hObject, eventdata, handles, varargin)

% Initialisierungen
handles.punkte = [];        % koordinaten der Punkte in Axis1
handles.cluster = [];       % koordinaten der Clusterpunkte
handles.lastCluster = [];   % koordinaten der letzten Clusterpunkte - zum plotten
handles.XMax = 10;          % maximum der x-achse
handles.YMax = 10;          % maximum der y-achse
handles.gridEinAus = 0;     % switch: grid ein(1)/aus(0)
handles.invEinAus = 0;      % switch: farben invertiert ein(1)/aus(0)
handles.U = [];             % speichert fr Ein-Schritt und n-Schritt
                            % U zur bergabe an den nchsten Schritt
handles.aktFehler = 0;      % speichert aktuellen Fehler fr Ein-Schritt 
                            % und n-Schritt
handles.schritt = 0;        % speichert den aktuellen "Zeit"-Schritt
handles.farbMatrix = farbMatrixBerechnen(handles); % farbtabelle neu berechnen
                           

% Choose default command line output for fuzzyCMeansGUI
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes fuzzyCMeansGUI wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = fuzzyCMeansGUI_OutputFcn(hObject, eventdata, handles) 
% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in butPKtZufall.
function butPKtZufall_Callback(hObject, eventdata, handles)

% zufallspunkte erzeugen
anzPunkte = str2num(get(handles.textAnzPunkte, 'String'));
XMax = get(handles.sliderXMax, 'Value');
YMax = get(handles.sliderYMax, 'Value');
zufallspunkte = rand(2, anzPunkte);
zufallspunkte(1,:) = zufallspunkte(1,:) * XMax;
zufallspunkte(2,:) = zufallspunkte(2,:) * YMax;
handles.punkte = [handles.punkte zufallspunkte];

% cluster loeschen, neue Punkte => neue berechnung
handles.cluster = [];
handles.lastCluster = [];

% zugehoerigkeitsmatrix U fr 1-Schritt und n-Schritt anpassen
[blabla, anzPunkte] = size(handles.punkte);
anzCluster = str2num(get(handles.textAnzCluster, 'String'));
handles.U = rand(anzCluster, anzPunkte);
handles.U = normiereU(handles.U, anzPunkte);

% Ausgabe
plotPunkte(handles.axis1, handles);
guidata(hObject, handles);


% --- Executes on slider movement.
function sliderAnzPunkte_Callback(hObject, eventdata, handles)
val = get(hObject, 'Value');
val = val - mod(val,1);
set(handles.textAnzPunkte, 'string', num2str(val));
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function sliderAnzPunkte_CreateFcn(hObject, eventdata, handles)
usewhitebg = 1;
if usewhitebg
    set(hObject,'BackgroundColor',[.9 .9 .9]);
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on button press in butPktMaus.
function butPktMaus_Callback(hObject, eventdata, handles)

% mauspunkte einlesen
[x, y] = getpts();
mausPunkte = [];
for i = 1:size(x),
    if (x(i) >= 0) && (y(i) >= 0),
        mausPunkte = [mausPunkte [x(i); y(i)]];
    end
end
handles.punkte = [handles.punkte mausPunkte];

% cluster loeschen, neue Punkte => neue berechnung
handles.cluster = [];
handles.lastCluster = [];

% U fr 1-Schritt und n-Schritt anpassen
[blabla, anzPunkte] = size(handles.punkte);
anzCluster = str2num(get(handles.textAnzCluster, 'String'));
handles.U = rand(anzCluster, anzPunkte);
handles.U = normiereU(handles.U, anzPunkte);

% Ausgabe
plotPunkte(handles.axis1, handles);
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function anzCluster_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes during object creation, after setting all properties.
function xMax_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes during object creation, after setting all properties.
function editYMax_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on button press in butNSchritte.
function butNSchritte_Callback(hObject, eventdata, handles)

% n mal fuzzy-c-means-schleife ausfhren
[dimZ, dimS] = size(handles.punkte);
if dimS > 1,
    % anzahl der cluster auf hchstens anzahl der punkte beschrnken
    anzCluster = testeAnzClusterPunkte(hObject, handles);

    M = handles.punkte;
    c = anzCluster;
    alpha = str2num(get(handles.textAlpha, 'String'));

    handles.lastCluster = [handles.lastCluster handles.cluster];
    NSchritte = str2num(get(handles.editNSchritte, 'String'));
    for i = 1:NSchritte,
        [handles.cluster, handles.U, handles.aktFehler] ...
                        = fuzzyCMeans_Schritt(M, c, alpha, handles.U);
        set(handles.textAktSchritt, 'String', ...
                        num2str(1+str2num(get(handles.textAktSchritt, 'String'))));

    end

    % ausgabe
    set(handles.textAktFehler, 'String', num2str(handles.aktFehler));
    plotPunkte(handles.axis1, handles);
    plotCluster(handles.axis1, handles);
    konsolenAusgabe(handles);
else
    disp('========= FEHLER =========');
    disp('Anzahl der Punkte ausser Wertebereich: 2 <= anzPunkte !');
    disp(' ');
end


% --- Executes during object creation, after setting all properties.
function editNSchritte_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes during object creation, after setting all properties.
function editError_CreateFcn(hObject, eventdata, handles)
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on slider movement.
function sliderAnzCluster_Callback(hObject, eventdata, handles)

% anzeige fr wert erneuern
val = get(hObject, 'Value');
val = val - mod(val,1);
set(handles.textAnzCluster, 'string', num2str(val));

handles.cluster = [];
handles.lastCluster = [];

% zugehoerigkeitsmatrix U fr 1-Schritt und n-Schritt anpassen
[blabla, anzPunkte] = size(handles.punkte);
anzCluster = str2num(get(handles.textAnzCluster, 'String'));
handles.U = rand(anzCluster, anzPunkte);
handles.U = normiereU(handles.U, anzPunkte);

guidata(hObject, handles);

% farbtabelle neu berechnen
handles.farbMatrix = farbMatrixBerechnen(handles);

plotPunkte(handles.axis1, handles);



% --- Executes during object creation, after setting all properties.
function sliderAnzCluster_CreateFcn(hObject, eventdata, handles)
usewhitebg = 1;
if usewhitebg
    set(hObject,'BackgroundColor',[.9 .9 .9]);
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on slider movement.
function sliderAlpha_Callback(hObject, eventdata, handles)

% anzeige fr wert erneuern
val = get(hObject, 'Value');
val = val - mod(val,.01);
set(handles.textAlpha, 'string', num2str(val));
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function sliderAlpha_CreateFcn(hObject, eventdata, handles)
usewhitebg = 1;
if usewhitebg
    set(hObject,'BackgroundColor',[.9 .9 .9]);
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on slider movement.
function sliderXMax_Callback(hObject, eventdata, handles)

% anzeige fr wert erneuern
val = get(hObject, 'Value');
val = val - mod(val,.5);
set(handles.textXMax, 'string', num2str(val));

% koordinatensystem anpassen
handles.XMax = val;
set(handles.axis1, 'XLim', [0.0 handles.XMax]);
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function sliderXMax_CreateFcn(hObject, eventdata, handles)
usewhitebg = 1;
if usewhitebg
    set(hObject,'BackgroundColor',[.9 .9 .9]);
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on slider movement.
function sliderYMax_Callback(hObject, eventdata, handles)

% anzeige fr wert erneuern
val = get(hObject, 'Value');
val = val - mod(val,.5);
set(handles.textYMax, 'string', num2str(val));

% koordinatensystem anpassen
handles.YMax = val;
set(handles.axis1, 'YLim', [0.0 handles.YMax]);
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function sliderYMax_CreateFcn(hObject, eventdata, handles)
usewhitebg = 1;
if usewhitebg
    set(hObject,'BackgroundColor',[.9 .9 .9]);
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on button press in butDelPunkte.
function butDelPunkte_Callback(hObject, eventdata, handles)

% punkte loeschen
handles.punkte = [];

% cluster loeschen
handles.cluster = [];
handles.lastCluster = [];

% U fr 1-Schritt und n-Schritt anpassen
handles.U = [];

% Ausgabe
if handles.invEinAus == 0, plot(0, 0, 'w');
else plot(0, 0, 'black'); end;
if handles.gridEinAus == 1, grid on; else grid off; end;
if handles.invEinAus == 1, set(handles.axis1, 'Color', [0 0 0]); end
set(handles.axis1, 'XLim', [0.0 handles.XMax]);
set(handles.axis1, 'YLim', [0.0 handles.YMax]);
guidata(hObject, handles);


% --- Executes on button press in butGitterEinAus.
function butGitterEinAus_Callback(hObject, eventdata, handles)

% gitter ein/aus schalten
if handles.gridEinAus == 0,
    grid on;
    handles.gridEinAus = 1;
    set(handles.butGitterEinAus, 'String', 'Gitter aus');
else
    grid off;
    handles.gridEinAus = 0;
    set(handles.butGitterEinAus, 'String', 'Gitter ein');
end
guidata(hObject, handles);


% --- Executes on button press in butDelCluster.
function butDelCluster_Callback(hObject, eventdata, handles)

% loesche clusterjkoordinaten
handles.cluster = [];
handles.lastCluster = [];
plotPunkte(handles.axis1, handles);


% --- Executes during object creation, after setting all properties.
function sliderAnzSchritte_CreateFcn(hObject, eventdata, handles)
usewhitebg = 1;
if usewhitebg
    set(hObject,'BackgroundColor',[.9 .9 .9]);
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on slider movement.
function sliderError_Callback(hObject, eventdata, handles)

% anzeigewert erneuern
val = get(hObject, 'Value');
val = val - mod(val,.001);
set(handles.editError, 'string', num2str(val));
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function sliderError_CreateFcn(hObject, eventdata, handles)
usewhitebg = 1;
if usewhitebg
    set(hObject,'BackgroundColor',[.9 .9 .9]);
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on slider movement.
function sliderNSchritte_Callback(hObject, eventdata, handles)

% anzeigewert erneuern
val = get(hObject, 'Value');
val = val - mod(val,1);
set(handles.editNSchritte, 'string', num2str(val));
guidata(hObject, handles);


% --- Executes on button press in but1Schritt.
function but1Schritt_Callback(hObject, eventdata, handles)

% einmal fuzzy-c-means-schleife laufen lassen
[dimZ, dimS] = size(handles.punkte);
if dimS > 1,
    % anzahl der cluster auf hoechstens anzahl der punkte beschraenken
    anzCluster = testeAnzClusterPunkte(hObject, handles);

    M = handles.punkte;
    c = anzCluster;
    alpha = str2num(get(handles.textAlpha, 'String'));

    handles.lastCluster = [handles.lastCluster handles.cluster];
    [handles.cluster, handles.U, handles.aktFehler] ...
                    = fuzzyCMeans_Schritt(M, c, alpha, handles.U);

    set(handles.textAktSchritt, 'String', ...
                    num2str(1+str2num(get(handles.textAktSchritt, 'String'))));
                
    % ausgabe
    set(handles.textAktFehler, 'String', num2str(handles.aktFehler));
    plotPunkte(handles.axis1, handles);
    plotCluster(handles.axis1, handles);
    konsolenAusgabe(handles);
else
    disp('========= FEHLER =========');
    disp('Anzahl der Punkte ausser Wertebereich: 2 <= anzPunkte !');
    disp(' ');
end


% --- Executes on button press in butError.
function butError_Callback(hObject, eventdata, handles)

% fuzzy-c-means-schleife bis abbruch laufen lassen
[dimZ, dimS] = size(handles.punkte);
if dimS > 1,
    % anzahl der cluster auf hchstens anzahl der punkte beschrnken
    anzCluster = testeAnzClusterPunkte(hObject, handles);

    M = handles.punkte;
    c = anzCluster;
    alpha = str2num(get(handles.textAlpha, 'String'));
    error = str2num(get(handles.editError, 'String'));

    [handles.cluster, handles.U, handles.aktFehler, schritte] ...
                    = fuzzyCMeans_Error(M, c, alpha, error);

    handles.lastCluster = [];

    % ausgabe
    set(handles.textAktFehler, 'String', num2str(handles.aktFehler));
    plotPunkte(handles.axis1, handles);
    plotCluster(handles.axis1, handles);
    konsolenAusgabe(handles);
    
    set(handles.textAktSchritt, 'String', num2str(schritte));
    guidata(hObject, handles);
else
    disp('========= FEHLER =========');
    disp('Anzahl der Punkte ausser Wertebereich: 2 <= anzPunkte !');
    disp(' ');
end


% --- Executes on button press in butDelMatrixU.
function butDelMatrixU_Callback(hObject, eventdata, handles)

% Clusterpunkte loeschen
handles.cluster = [];
handles.lastCluster = [];

% U fr 1-Schritt und n-Schritt anpassen
[blabla, anzPunkte] = size(handles.punkte);
anzCluster = str2num(get(handles.textAnzCluster, 'String'));
handles.U = rand(anzCluster, anzPunkte);
handles.U = normiereU(handles.U, anzPunkte);
guidata(hObject, handles);

set(handles.textAktSchritt, 'String', '0');
set(handles.textAktFehler, 'String', '0');
handles.cluster = [];
guidata(hObject, handles);
plotPunkte(handles.axis1, handles);


% --- Executes on button press in butInvFarbe.
function butInvFarbe_Callback(hObject, eventdata, handles)
if handles.invEinAus == 0,
    handles.invEinAus = 1;
    set(handles.axis1, 'Color', [0 0 0]); 
else
    handles.invEinAus = 0;
    set(handles.axis1, 'Color', [1 1 1]); 
end    
guidata(hObject, handles);
plotPunkte(handles.axis1, handles);
plotCluster(handles.axis1, handles);


% --- Executes on button press in butPktspeichern.
function butPktSpeichern_Callback(hObject, eventdata, handles)

m = handles.punkte;
alpha = str2num(get(handles.textAlpha, 'String'));
c = str2num(get(handles.textAnzCluster, 'String'));
error = str2num(get(handles.editError, 'String'));

[dateiname pfad] = uiputfile('*.mat');
if dateiname ~= 0,
    save([pfad dateiname], 'm', 'alpha', 'c', 'error');
end


% --- Executes on button press in butPktLaden.
function butPktLaden_Callback(hObject, eventdata, handles)
[dateiname pfad] = uigetfile('*.mat');

if dateiname ~= 0,
    mat = load(fullfile(pfad,dateiname));
    handles.punkte = [handles.punkte mat.m];

    % anzeige von anzahl cluster aendern
    val = mat.c;
    val = val - mod(val,1);
    set(handles.sliderAnzCluster, 'Value', val);
    set(handles.textAnzCluster, 'String', num2str(val));

    % cluster loeschen
    handles.cluster = [];
    handles.lastCluster = [];

    % zugehoerigkeitsmatrix U fr 1-Schritt und n-Schritt anpassen
    [blabla, anzPunkte] = size(handles.punkte);
    anzCluster = str2num(get(handles.textAnzCluster, 'String'));
    handles.U = rand(anzCluster, anzPunkte);
    handles.U = normiereU(handles.U, anzPunkte);

    % anzeige von alpha erneuern
    val = mat.alpha;
    val = val - mod(val,.01);
    set(handles.sliderAlpha, 'Value', val);
    set(handles.textAlpha, 'string', num2str(val));

    % anzeige von error erneuern
    val = mat.error;
    val = val - mod(val,.001);
    set(handles.sliderError, 'Value', val);
    set(handles.editError, 'string', num2str(val));


    guidata(hObject, handles);

    % farbtabelle neu berechnen
    handles.farbMatrix = farbMatrixBerechnen(handles);

    plotPunkte(handles.axis1, handles);
end


% --- Executes on button press in butClose.
function butClose_Callback(hObject, eventdata, handles)
close(gcf);


% --- Executes on button press in butInfo.
function butInfo_Callback(hObject, eventdata, handles)
help fuzzyCMeansGUI;


% --- plottet clusterzentren
function plotCluster(hObject, handles)

% ist dann leichter die linien zu zeichnen
clusterFuerRechnung = [handles.lastCluster handles.cluster];

hold on;
%anzCluster = str2num(get(handles.textAnzCluster, 'String'));
[zwei, anzCluster]     = size(handles.cluster);
[zwei, anzLastCluster] = size(handles.lastCluster);
for i = 1:anzLastCluster,
    
    % schon vorhandene clusterpunkte plotten
    clusterZahl = mod(i,anzCluster);
    if clusterZahl == 0, clusterZahl = anzCluster; end
    
    % farbe bestimmen
    farbe = handles.farbMatrix(clusterZahl,:);
    if handles.invEinAus == 1, farbe = [1 1 1] - farbe; end
    
    plot(clusterFuerRechnung(1,i), clusterFuerRechnung(2,i), 'o', ...
        'Color', farbe);
    
    % linien zwischen clusterpunkte plotten
    xVektor = [clusterFuerRechnung(1,i) clusterFuerRechnung(1,i+anzCluster)];
    yVektor = [clusterFuerRechnung(2,i) clusterFuerRechnung(2,i+anzCluster)];
    plot(xVektor, yVektor, 'Color', farbe);
end

% aktuelle cluster als kreis plotten
for i = 1:anzCluster,
    
    % farbe bestimmen
    farbe = handles.farbMatrix(i,:);
    if handles.invEinAus == 1, farbe = [1 1 1] - farbe; end
    
    plot(handles.cluster(1,i), handles.cluster(2,i), 'o', 'Color', farbe);
end
hold off;

if handles.gridEinAus == 1, grid on; else grid off; end;
if handles.invEinAus == 1, set(handles.axis1, 'Color', [0 0 0]); end
set(handles.axis1, 'XLim', [0.0 handles.XMax]);
set(handles.axis1, 'YLim', [0.0 handles.YMax]);
guidata(hObject, handles);


% --- plottet punkte
function plotPunkte(hObject, handles)

% falls punkte existieren, plotten als x
if length(handles.punkte) > 0,
    
    [zwei, anzPunkte] = size(handles.punkte);
    for i = 1:anzPunkte,
        
        if i == 2, hold on; end
        
        % farbe berechnen
        [vec, ind] = sort(handles.U(:,i), 'descend');
        grauton = vec(1) - vec(2); % gibt an, wie sehr der punkt
                                   % zu einem cluster gehoert
                                   % skaliert farbwerte, da grauton
                                   % zwischn 0 und 1 liegt
        % in ind(1) steht dann, zu welchem cluster er ueberwiegend gehoert
        
        farbe = grauton*handles.farbMatrix(ind(1),:);
        if handles.invEinAus == 1, farbe = [1 1 1] - farbe; end
        
        plot(handles.punkte(1,i), handles.punkte(2,i), '*', 'Color', farbe);
    end
    hold off;
    
end
if handles.gridEinAus == 1, grid on; else grid off; end;
if handles.invEinAus == 1, set(handles.axis1, 'Color', [0 0 0]); end
set(handles.axis1, 'XLim', [0.0 handles.XMax]);
set(handles.axis1, 'YLim', [0.0 handles.YMax]);
guidata(hObject, handles);


% ==== normiert eine Matrix U, so dass die Eintrge einer
%      Spalte als Summe 1 ergeben
function nU = normiereU(U, anzPunkte)
for i = 1:anzPunkte,
    summe = sum(U(:,i));
    U(:,i) = U(:,i) / summe;
end
nU = U;


% --- gibt diverse Werte auf der Konsole aus
function konsolenAusgabe(handles)
if (get(handles.checkAktSchritt, 'Value') == get(handles.checkAktSchritt, 'Max'))...
|| (get(handles.checkAktFehler, 'Value') == get(handles.checkAktFehler, 'Max'))...
|| (get(handles.checkCluster, 'Value') == get(handles.checkCluster, 'Max'))...
|| (get(handles.checkPunkte, 'Value') == get(handles.checkPunkte, 'Max'))...
|| (get(handles.checkMatrix, 'Value') == get(handles.checkMatrix, 'Max')),
    disp(' ');
    disp('=================================================');
    disp(' ');
end
if (get(handles.checkAktSchritt, 'Value') == get(handles.checkAktSchritt, 'Max')),
    aktuellerSchritt = str2num(get(handles.textAktSchritt, 'String'))
end
if (get(handles.checkAktFehler, 'Value') == get(handles.checkAktFehler, 'Max')),
    aktuellerFehler = handles.aktFehler
end
if (get(handles.checkCluster, 'Value') == get(handles.checkCluster, 'Max')),
    Clusterkoordinaten = handles.cluster
end
if (get(handles.checkPunkte, 'Value') == get(handles.checkPunkte, 'Max')),
    Punktkoordinaten = handles.punkte
end
if (get(handles.checkMatrix, 'Value') == get(handles.checkMatrix, 'Max')),
    ZugehoerigkeitsmatrixU = handles.U
end


% --- beschrnkt bei bedarf die anzahl der cluster 
%     auf die anzahl der punkte
function anzCluster = testeAnzClusterPunkte(hObject, handles)
anzCluster = str2num(get(handles.textAnzCluster, 'String'));
[Zwei, anzPunkte] = size(handles.punkte);
if anzCluster > anzPunkte,
    disp('========= FEHLER =========');
    disp('Anzahl der Cluster wurde auf Anzahl der Punkte reduziert!');
    disp(' ');
    anzCluster = anzPunkte;
    set(handles.sliderAnzCluster, 'Value', anzPunkte);
    set(handles.textAnzCluster, 'String', num2str(anzPunkte));
end %if
guidata(hObject, handles);


% --- bestimme zu einer bestimmen anzahl cluster
%     eine (#Cluster,3)-farbmatrix
function farbMatrix = farbMatrixBerechnen(handles)
anzCluster = str2num(get(handles.textAnzCluster, 'String'));
delta = 4 / anzCluster;
for i = 1:anzCluster,
    wert = 0 + (i-1)*delta;
    if wert >= 0, rot = wert-0; gruen = 1;      blau = 0;      end
    if wert >= 1, rot = 1;      gruen = 2-wert; blau = 0;      end
    if wert >= 2, rot = 1;      gruen = 0;      blau = wert-2; end
    if wert >= 3, rot = 4-wert; gruen = 0;      blau = 1;      end
    farbMatrix(i,:) = [rot gruen blau];
end
    

function anzCluster_Callback(hObject, eventdata, handles)
% NIX

function xMax_Callback(hObject, eventdata, handles)
% NIX

function editYMax_Callback(hObject, eventdata, handles)
% NIX

function editNSchritte_Callback(hObject, eventdata, handles)
% NIX

function editError_Callback(hObject, eventdata, handles)
% NIX

function sliderAnzSchritte_Callback(hObject, eventdata, handles)
% NIX

function checkCluster_Callback(hObject, eventdata, handles)
% NIX

function checkPunkte_Callback(hObject, eventdata, handles)
% NIX

function checkAktFehler_Callback(hObject, eventdata, handles)
% NIX

function checkMatrix_Callback(hObject, eventdata, handles)
% NIX

function checkAktSchritt_Callback(hObject, eventdata, handles)
% NIX

% --- Executes during object creation, after setting all properties.
function figure1_CreateFcn(hObject, eventdata, handles)
% NX

